from .api_bad_request_error import ApiBadRequestError
from .api_error import ApiError
from .app import App
from .app_balance import AppBalance
from .app_create_withdrawal_dto import AppCreateWithdrawalDto
from .app_dto import AppDto
from .available_coins import AvailableCoins
from .available_coins_data import AvailableCoinsData
from .cheque import Cheque
from .coin_dto import CoinDto
from .create_transfer_dto import CreateTransferDto
from .delete_response_dto import DeleteResponseDto
from .full_invoice_dto import FullInvoiceDto
from .full_invoice_response_dto import FullInvoiceResponseDto
from .invoice import Invoice
from .invoice_dto import InvoiceDto
from .pagination import Pagination
from .pay_invoice_dto import PayInvoiceDto
from .pay_invoice_stat_dto import PayInvoiceStatDto
from .property_error import PropertyError
from .response_dto import ResponseDto
from .response_multi_dto import ResponseMultiDto
from .set import Set
from .short_cheque_dto import ShortChequeDto
from .subscription import Subscription
from .subscription_interval import SubscriptionInterval
from .subscription_interval_dto import SubscriptionIntervalDto
from .subscription_resource import SubscriptionResource
from .tg_resource import TgResource
from .transfer_dto import TransferDto
from .update import Update
from .update_cheque_dto import UpdateChequeDto
from .user_subscription import UserSubscription
from .user_subscription_transaction import UserSubscriptionTransaction
from .version import Version
from .webhook_dto import WebhookDto
from .withdraw_fee_dto import WithdrawFeeDto
from .withdraw_fee_network_dto import WithdrawFeeNetworkDto
from .withdraw_fee_network_fee_dto import WithdrawFeeNetworkFeeDto
from .withdrawal_coin_fees_response_dto import WithdrawalCoinFeesResponseDto
from .withdrawal_coin_response_dto import WithdrawalCoinResponseDto
from .withdrawal_dto import WithdrawalDto

__all__ = [
    "ApiBadRequestError",
    "ApiError",
    "App",
    "AppBalance",
    "AppCreateWithdrawalDto",
    "AppDto",
    "AvailableCoins",
    "AvailableCoinsData",
    "Cheque",
    "CoinDto",
    "CreateTransferDto",
    "DeleteResponseDto",
    "FullInvoiceDto",
    "FullInvoiceResponseDto",
    "Invoice",
    "InvoiceDto",
    "Pagination",
    "PayInvoiceDto",
    "PayInvoiceStatDto",
    "PropertyError",
    "ResponseDto",
    "ResponseMultiDto",
    "Set",
    "ShortChequeDto",
    "Subscription",
    "SubscriptionInterval",
    "SubscriptionIntervalDto",
    "SubscriptionResource",
    "TgResource",
    "TransferDto",
    "Update",
    "UpdateChequeDto",
    "UserSubscription",
    "UserSubscriptionTransaction",
    "Version",
    "WebhookDto",
    "WithdrawFeeDto",
    "WithdrawFeeNetworkDto",
    "WithdrawFeeNetworkFeeDto",
    "WithdrawalCoinFeesResponseDto",
    "WithdrawalCoinResponseDto",
    "WithdrawalDto",
]



from typing import Optional, Union

from pydantic import Field
from stollen.enums import HTTPMethod

from ..types import TransferDto
from .base import PayXRocketMethod


class Transfer(
    PayXRocketMethod[TransferDto],
    http_method=HTTPMethod.POST,
    api_method="/app/transfer",
    returning=TransferDto,
    response_data_key=["data"],
):
    """
    Make transfer of funds to another user
    """

    tg_user_id: Union[int, float] = Field(serialization_alias="tgUserId")
    """Telegram user ID. If we dont have this user in DB, we will fail transaction with error: 400 - User not found"""
    currency: str
    """Currency of transfer, info /currencies/available"""
    amount: Union[int, float]
    """Transfer amount. 9 decimal places, others cut off"""
    transfer_id: str = Field(serialization_alias="transferId")
    """Unique transfer ID in your system to prevent double spends"""
    description: Optional[str] = None
    """Transfer description"""



from datetime import datetime
from typing import Optional, Union

from pydantic import Field

from .base import TradeXRocketObject
from .exchange_order_balance_dto import ExchangeOrderBalanceDto
from .exchange_order_filled_balance_dto import ExchangeOrderFilledBalanceDto
from .exchange_order_last_execution_dto import ExchangeOrderLastExecutionDto


class ExchangeOrderDto(TradeXRocketObject):
    order_id: Union[int, float] = Field(alias="orderId")
    """Order ID"""
    pair: str
    """Pair name"""
    type: str
    """Order type"""
    execute_type: str = Field(alias="executeType")
    """Order execute type"""
    status: str
    """Order status"""
    rate: Union[int, float]
    """Rate"""
    main_amount: Union[int, float] = Field(alias="mainAmount")
    """Init main amount"""
    secondary_amount: Union[int, float] = Field(alias="secondaryAmount")
    """Init secondary amount"""
    filled: Union[int, float]
    """Filled percent"""
    balance: ExchangeOrderBalanceDto
    """Now order balance"""
    filled_balance: ExchangeOrderFilledBalanceDto = Field(alias="filledBalance")
    """Filled order balance"""
    last_execution: Optional[ExchangeOrderLastExecutionDto] = Field(
        default=None, alias="lastExecution"
    )
    """Last execution, for webhook only"""
    created: datetime
    """Created date"""
    closed: datetime
    """Closed date"""



from typing import Union

from pydantic import Field
from stollen.enums import HTTPMethod

from ..types import OrderEstimateDto
from .base import TradeXRocketMethod


class GetOrderEstimate(
    TradeXRocketMethod[OrderEstimateDto],
    http_method=HTTPMethod.POST,
    api_method="/orders/estimate",
    returning=OrderEstimateDto,
    response_data_key=["data"],
):
    """
    Returns order estimate
    """

    pair_id: Union[int, float] = Field(serialization_alias="pairId")
    """Pair ID"""
    type: str
    """Order type"""
    execute_type: str = Field(serialization_alias="executeType")
    """Order execute type"""
    rate: Union[int, float]
    """Order rate"""
    amount: Union[int, float]
    """Amount"""
    currency: str
    """Amount currency"""



from typing import Optional, Union

from pydantic import Field

from .base import PayXRocketObject


class TransferDto(PayXRocketObject):
    id: Union[int, float]
    """Transfer ID"""
    tg_user_id: Union[int, float] = Field(alias="tgUserId")
    """Telegram user ID"""
    currency: str
    """Currency code"""
    amount: Union[int, float]
    """Transfer amount. 9 decimal places, others cut off"""
    description: Optional[str] = None
    """Transfer description"""



from typing import Union

from pydantic import Field

from .base import TradeXRocketObject


class ExchangeOrderFilledBalanceDto(TradeXRocketObject):
    main_amount: Union[int, float] = Field(alias="mainAmount")
    """Current base amount"""
    secondary_amount: Union[int, float] = Field(alias="secondaryAmount")
    """Current quote amount"""



from typing import Optional

from .base import PayXRocketObject
from .property_error import PropertyError


class ApiBadRequestError(PayXRocketObject):
    success: bool
    message: str
    errors: Optional[list[PropertyError]] = None



from typing import Optional, Union

from pydantic import Field

from .base import PayXRocketObject


class CreateTransferDto(PayXRocketObject):
    tg_user_id: Union[int, float] = Field(alias="tgUserId")
    """Telegram user ID. If we dont have this user in DB, we will fail transaction with error: 400 - User not found"""
    currency: str
    """Currency of transfer, info /currencies/available"""
    amount: Union[int, float]
    """Transfer amount. 9 decimal places, others cut off"""
    transfer_id: str = Field(alias="transferId")
    """Unique transfer ID in your system to prevent double spends"""
    description: Optional[str] = None
    """Transfer description"""



from datetime import datetime
from typing import Union

from pydantic import Field

from .base import TradeXRocketObject


class TimeSerieDto(TradeXRocketObject):
    open_rate: Union[int, float] = Field(alias="openRate")
    """Rate open"""
    close_rate: Union[int, float] = Field(alias="closeRate")
    """Rate close"""
    min_rate: Union[int, float] = Field(alias="minRate")
    """Min rate"""
    max_rate: Union[int, float] = Field(alias="maxRate")
    """Max rate"""
    open_time: datetime = Field(alias="openTime")
    """Open time"""
    close_time: datetime = Field(alias="closeTime")
    """Close time"""
    volume: Union[int, float]
    """Volume"""
    quote_volume: Union[int, float] = Field(alias="quoteVolume")
    """Quote volume"""



from stollen.enums import HTTPMethod

from ..types import Subscription
from .base import PayXRocketMethod


class GetSubscription(
    PayXRocketMethod[Subscription],
    http_method=HTTPMethod.GET,
    api_method="/subscriptions/{subscription_id}",
    returning=Subscription,
    response_data_key=["data"],
):
    """
    Get subscription info
    """

    subscription_id: str



from typing import Optional

from stollen.enums import HTTPMethod

from .base import PayXRocketMethod


class DeleteSubscription(
    PayXRocketMethod[Optional[bool]],
    http_method=HTTPMethod.DELETE,
    api_method="/subscriptions/{subscription_id}",
    returning=Optional[bool],
    response_data_key=["success"],
):
    """
    Delete subscription
    """

    subscription_id: str



from datetime import datetime
from typing import Union

from pydantic import Field

from .base import TradeXRocketObject


class LastTradesResponseDto(TradeXRocketObject):
    amount: Union[int, float]
    """Amount in quote token"""
    order_time: datetime = Field(alias="orderTime")
    """Order time"""
    price: Union[int, float]
    """Order price"""
    quantity: Union[int, float]
    """Quantity in base token"""
    side: str
    """Volume"""



from typing import Optional, Union

from pydantic import Field

from .base import TradeXRocketObject


class WithdrawalResponseDto(TradeXRocketObject):
    network: str
    """Network code"""
    address: str
    """Withdrawal address"""
    currency: str
    """Currency code"""
    amount: Union[int, float]
    """Withdrawal amount. 9 decimal places, others cut off"""
    withdrawal_id: str = Field(alias="withdrawalId")
    """Unique withdrawal ID in your system to prevent double spends"""
    comment: Optional[str] = None
    """Withdrawal comment"""
    status: Optional[str] = None
    """Withdrawal status"""
    tx_hash: Optional[str] = Field(default=None, alias="txHash")
    """Withdrawal TX hash. Provided only after withdrawal"""
    tx_link: Optional[str] = Field(default=None, alias="txLink")
    """Withdrawal TX link. Provided only after withdrawal"""



from stollen.enums import HTTPMethod

from ..types import RateCryptoCryptoDto
from .base import TradeXRocketMethod


class GetCryptoRateInCrypto(
    TradeXRocketMethod[RateCryptoCryptoDto],
    http_method=HTTPMethod.GET,
    api_method="/rates/crypto/{base}/{quote}",
    returning=RateCryptoCryptoDto,
    response_data_key=["data"],
):
    """
    Returns rate crypto currency in crypto
    """

    base: str
    quote: str



from datetime import datetime

from .base import PayXRocketObject


class WebhookDto(PayXRocketObject):
    type: str
    """type of webhook sent"""
    timestamp: datetime
    """When webhook was sent"""



from typing import Optional

from stollen.enums import HTTPMethod

from ..types import WithdrawalCoinResponseDto
from .base import TradeXRocketMethod


class GetWithdrawalFees(
    TradeXRocketMethod[list[WithdrawalCoinResponseDto]],
    http_method=HTTPMethod.GET,
    api_method="/account/withdrawal/fees",
    returning=list[WithdrawalCoinResponseDto],
    response_data_key=["data"],
):
    """
    Returns withdrawal fees
    """

    currency: Optional[str] = None



from .account_balance_dto import AccountBalanceDto
from .account_balances_dto import AccountBalancesDto
from .api_create_withdrawal_dto import ApiCreateWithdrawalDto
from .cancel_order_response_dto import CancelOrderResponseDto
from .create_order_request_dto import CreateOrderRequestDto
from .crypto_currencies_response_dto import CryptoCurrenciesResponseDto
from .exchange_order_balance_dto import ExchangeOrderBalanceDto
from .exchange_order_dto import ExchangeOrderDto
from .exchange_order_filled_balance_dto import ExchangeOrderFilledBalanceDto
from .exchange_order_last_execution_dto import ExchangeOrderLastExecutionDto
from .fiat_currencies_dto import FiatCurrenciesDto
from .last_trades_response import LastTradesResponse
from .last_trades_response_dto import LastTradesResponseDto
from .order_book_full_dto import OrderBookFullDto
from .order_book_order_dto import OrderBookOrderDto
from .order_estimate_dto import OrderEstimateDto
from .order_estimate_request_dto import OrderEstimateRequestDto
from .pagination import Pagination
from .pair_dto import PairDto
from .rate_crypto_crypto_dto import RateCryptoCryptoDto
from .rate_crypto_fiat_dto import RateCryptoFiatDto
from .response_multi_dto import ResponseMultiDto
from .response_paginated_dto import ResponsePaginatedDto
from .response_single_dto import ResponseSingleDto
from .time_serie_dto import TimeSerieDto
from .time_series_dto import TimeSeriesDto
from .user_fees_response import UserFeesResponse
from .user_fees_response_dto import UserFeesResponseDto
from .version_response_dto import VersionResponseDto
from .webhook_dto import WebhookDto
from .withdraw_fee_network_fee_dto import WithdrawFeeNetworkFeeDto
from .withdrawal_coin_fees_response_dto import WithdrawalCoinFeesResponseDto
from .withdrawal_coin_response_dto import WithdrawalCoinResponseDto
from .withdrawal_fees_request_dto import WithdrawalFeesRequestDto
from .withdrawal_fees_response import WithdrawalFeesResponse
from .withdrawal_response_dto import WithdrawalResponseDto

__all__ = [
    "AccountBalanceDto",
    "AccountBalancesDto",
    "ApiCreateWithdrawalDto",
    "CancelOrderResponseDto",
    "CreateOrderRequestDto",
    "CryptoCurrenciesResponseDto",
    "ExchangeOrderBalanceDto",
    "ExchangeOrderDto",
    "ExchangeOrderFilledBalanceDto",
    "ExchangeOrderLastExecutionDto",
    "FiatCurrenciesDto",
    "LastTradesResponse",
    "LastTradesResponseDto",
    "OrderBookFullDto",
    "OrderBookOrderDto",
    "OrderEstimateDto",
    "OrderEstimateRequestDto",
    "Pagination",
    "PairDto",
    "RateCryptoCryptoDto",
    "RateCryptoFiatDto",
    "ResponseMultiDto",
    "ResponsePaginatedDto",
    "ResponseSingleDto",
    "TimeSerieDto",
    "TimeSeriesDto",
    "UserFeesResponse",
    "UserFeesResponseDto",
    "VersionResponseDto",
    "WebhookDto",
    "WithdrawFeeNetworkFeeDto",
    "WithdrawalCoinFeesResponseDto",
    "WithdrawalCoinResponseDto",
    "WithdrawalFeesRequestDto",
    "WithdrawalFeesResponse",
    "WithdrawalResponseDto",
]



from typing import Union

from pydantic import Field

from .app_balance import AppBalance
from .base import PayXRocketObject


class App(PayXRocketObject):
    name: str
    """Name of current app"""
    fee_percents: Union[int, float] = Field(alias="feePercents")
    """Fee for incoming transactions"""
    balances: list[AppBalance]



from stollen.enums import HTTPMethod

from ..types import FullInvoiceDto
from .base import PayXRocketMethod


class GetInvoice(
    PayXRocketMethod[FullInvoiceDto],
    http_method=HTTPMethod.GET,
    api_method="/tg-invoices/{id}",
    returning=FullInvoiceDto,
    response_data_key=["data"],
):
    """
    Get invoice info
    """

    id: str



from datetime import datetime
from typing import Union

from pydantic import Field

from .base import PayXRocketObject
from .user_subscription_transaction import UserSubscriptionTransaction


class UserSubscription(PayXRocketObject):
    subscription_id: Union[int, float] = Field(alias="subscriptionId")
    """Subscription id for this payment"""
    subscription_code: str = Field(alias="subscriptionCode")
    """Subscription code for this payment"""
    user_id: Union[int, float] = Field(alias="userId")
    """TG user ID who pay subscription"""
    amount: Union[int, float]
    """Sum all payments which added to app balance"""
    currency: str
    """Subscription currency"""
    interval: str
    """Payed interval"""
    ref_fee: Union[int, float] = Field(alias="refFee")
    """Sum all referral rewards"""
    is_ref_pay: bool = Field(alias="isRefPay")
    """This payment by ref link"""
    total_amount: Union[int, float] = Field(alias="totalAmount")
    """Sum all payments which user pay"""
    payment_start: datetime = Field(alias="paymentStart")
    """When subscribe start"""
    payment_end: datetime = Field(alias="paymentEnd")
    """When subscribe ends"""
    auto_renewal: bool = Field(alias="autoRenewal")
    """Is auto renewal"""
    transactions: list[UserSubscriptionTransaction]
    """Payments for this user"""



from typing import Union

from pydantic import Field

from .base import TradeXRocketObject


class AccountBalanceDto(TradeXRocketObject):
    code: str
    """Crypto code"""
    amount: Union[int, float]
    """Amount crypto"""
    locked_in_orders: Union[int, float, None] = Field(alias="lockedInOrders", default=None)
    """Amount locked in exchange orders"""



from .base import PayXRocketObject


class PropertyError(PayXRocketObject):
    property: str
    """name of input field that cannot be processed"""
    error: str
    """text explaining what exactly wrong"""



from enum import Enum


class WebhookType(str, Enum):
    INVOICE_PAY = "invoicePay"
    SUBSCRIPTION_PAY = "subscriptionPay"
    SUBSCRIPTION_END = "subscriptionEnd"
    EXCHANGE_ORDER_COMPLETE = "exchangeOrderComplete"



from pydantic import Field

from .base import PayXRocketObject
from .withdraw_fee_network_fee_dto import WithdrawFeeNetworkFeeDto


class WithdrawFeeNetworkDto(PayXRocketObject):
    network_code: str = Field(alias="networkCode")
    """Network code for withdraw"""
    fee_withdraw: WithdrawFeeNetworkFeeDto = Field(alias="feeWithdraw")
    """Withdraw fee params"""



from typing import Optional, Union

from pydantic import Field

from .base import PayXRocketObject
from .withdraw_fee_dto import WithdrawFeeDto


class CoinDto(PayXRocketObject):
    currency: str
    """ID of currency, use in Rocket Pay Api"""
    name: str
    """Name of currency"""
    min_transfer: Union[int, float] = Field(alias="minTransfer")
    """Minimal amount for transfer"""
    min_cheque: Union[int, float] = Field(alias="minCheque")
    """Minimal amount for cheque"""
    min_invoice: Union[int, float] = Field(alias="minInvoice")
    """Minimal amount for invoice"""
    min_withdraw: Union[int, float] = Field(alias="minWithdraw")
    """Minimal amount for withdrawals"""
    fee_withdraw: Optional[WithdrawFeeDto] = Field(default=None, alias="feeWithdraw")
    """Fee for token withdrawals in main currency"""



from stollen.enums import HTTPMethod

from ..types import AccountBalanceDto
from .base import TradeXRocketMethod


class GetBalance(
    TradeXRocketMethod[AccountBalanceDto],
    http_method=HTTPMethod.GET,
    api_method="/account/balance/{coin}",
    returning=AccountBalanceDto,
    response_data_key=["data"],
):
    """
    Returns current balance of you account for coin
    """

    coin: str



from __future__ import annotations

import logging
from typing import Annotated, Any, Final

from fastapi import APIRouter, Body, Header, HTTPException, Request

from milkpay.xrocket.pay import PayXRocket
from milkpay.xrocket.pay.types import Update

router: Final[APIRouter] = APIRouter()
logger: Final[logging.Logger] = logging.getLogger(name=__name__)


@router.post("/webhook/xrocket")
async def accept_xrocket_payment(
    request: Request,
    rocket_pay_signature: Annotated[str, Header()],
    update: Annotated[Update, Body()],
) -> Any:
    xrocket: PayXRocket = request.app.state.xrocket
    if not xrocket.check_signature(
        body_text=(await request.body()).decode(),
        signature=rocket_pay_signature,
    ):
        raise HTTPException(status_code=401, detail="Invalid signature")
    logger.info("Received payment with id %d", update.data.id)



from typing import Optional, Union

from stollen.enums import HTTPMethod

from ..types import ExchangeOrderDto, Pagination
from .base import TradeXRocketMethod


class GetOrders(
    TradeXRocketMethod[Pagination[ExchangeOrderDto]],
    http_method=HTTPMethod.GET,
    api_method="/orders",
    returning=Pagination[ExchangeOrderDto],
    response_data_key=["data"],
):
    """
    Get list of exchange orders
    """

    limit: Optional[Union[int, float]] = None
    offset: Optional[Union[int, float]] = None
    only_active: bool



from typing import Optional

from stollen.enums import HTTPMethod

from .base import TradeXRocketMethod


class CancelOrder(
    TradeXRocketMethod[Optional[bool]],
    http_method=HTTPMethod.DELETE,
    api_method="/orders/{id}",
    returning=Optional[bool],
    response_data_key=["success"],
):
    """
    Cancel exchange order
    """

    id: str



from pydantic import Field

from .base import TradeXRocketObject
from .time_serie_dto import TimeSerieDto


class TimeSeriesDto(TradeXRocketObject):
    pair: str
    """Pair name"""
    time_series: list[TimeSerieDto] = Field(alias="timeSeries")
    """Rate open"""



from typing import Optional

from pydantic import Field
from stollen.enums import HTTPMethod

from ..types import Cheque
from .base import PayXRocketMethod


class EditCheque(
    PayXRocketMethod[Cheque],
    http_method=HTTPMethod.PUT,
    api_method="/multi-cheque/{id}",
    returning=Cheque,
    response_data_key=["data"],
):
    """
    Edit multi-cheque
    """

    id: str
    password: Optional[str] = None
    """Password for cheque"""
    description: Optional[str] = None
    """Description for cheque"""
    send_notifications: Optional[bool] = Field(
        default=None, serialization_alias="sendNotifications"
    )
    """Send notifications about activations"""
    enable_captcha: Optional[bool] = Field(default=None, serialization_alias="enableCaptcha")
    """Enable captcha"""
    telegram_resources_ids: Optional[list[str]] = Field(
        default=None, serialization_alias="telegramResourcesIds"
    )
    """IDs of telegram resources (groups, channels, private groups)"""
    for_premium: Optional[bool] = Field(default=None, serialization_alias="forPremium")
    """Only users with Telegram Premium can activate this cheque"""
    linked_wallet: Optional[bool] = Field(default=None, serialization_alias="linkedWallet")
    """Only users with linked wallet can activate this cheque"""
    disabled_languages: Optional[list[str]] = Field(
        default=None, serialization_alias="disabledLanguages"
    )
    """Disable languages"""



from datetime import datetime
from typing import Optional, Union

from pydantic import Field

from .base import PayXRocketObject


class InvoiceDto(PayXRocketObject):
    id: Union[int, float]
    """Invoice ID"""
    amount: Union[int, float]
    """Amount of invoice"""
    min_payment: Optional[Union[int, float]] = Field(default=None, alias="minPayment")
    """Min payment of invoice"""
    total_activations: Union[int, float] = Field(alias="totalActivations")
    """Total activations of invoice"""
    activations_left: Union[int, float] = Field(alias="activationsLeft")
    """Activations left of invoice"""
    description: Optional[str] = None
    """Invoice description"""
    hidden_message: Optional[str] = Field(default=None, alias="hiddenMessage")
    """Message that will be displayed after invoice payment"""
    payload: Optional[str] = None
    """Any data that is attached to invoice"""
    callback_url: Optional[str] = Field(default=None, alias="callbackUrl")
    """url that will be set for Return button after invoice is paid"""
    comments_enabled: Optional[bool] = Field(default=None, alias="commentsEnabled")
    """Allow comments for invoice"""
    currency: str
    created: Optional[datetime] = None
    """When invoice was created"""
    paid: Optional[datetime] = None
    """When invoice was paid"""
    status: str
    expired_in: Optional[Union[int, float]] = Field(default=None, alias="expiredIn")
    """Invoice expire time in seconds, max 1 day, 0 - none expired"""
    link: str



from typing import Optional

from pydantic import Field

from .base import PayXRocketObject


class TgResource(PayXRocketObject):
    telegram_id: str = Field(alias="telegramId")
    name: str
    username: Optional[str] = None



from stollen.enums import HTTPMethod

from ..types import RateCryptoFiatDto
from .base import TradeXRocketMethod


class GetCryptoRateInFiat(
    TradeXRocketMethod[RateCryptoFiatDto],
    http_method=HTTPMethod.GET,
    api_method="/rates/fiat/{crypto}/{fiat}",
    returning=RateCryptoFiatDto,
    response_data_key=["data"],
):
    """
    Returns rate crypto currency in fiat
    """

    crypto: str
    fiat: str



from typing import Union

from stollen.enums import HTTPMethod

from ..types import SubscriptionInterval
from .base import PayXRocketMethod


class CreateSubscriptionInterval(
    PayXRocketMethod[SubscriptionInterval],
    http_method=HTTPMethod.POST,
    api_method="/subscriptions/{subscription_id}/interval",
    returning=SubscriptionInterval,
    response_data_key=["data"],
):
    """
    Create subscription interval
    """

    subscription_id: str
    interval: str
    """Interval for subscription"""
    amount: Union[int, float]
    """Cost subscription for current interval in currency"""
    status: str
    """Status for subscription"""



from stollen.enums import HTTPMethod

from ..types import CryptoCurrenciesResponseDto
from .base import TradeXRocketMethod


class GetAvailableCryptoCurrencies(
    TradeXRocketMethod[CryptoCurrenciesResponseDto],
    http_method=HTTPMethod.GET,
    api_method="/rates/crypto/available",
    returning=CryptoCurrenciesResponseDto,
    response_data_key=["data"],
):
    """
    Returns available crypto currencies for exchange rates
    """



from typing import Union

from pydantic import Field

from .base import PayXRocketObject
from .withdrawal_coin_fees_response_dto import WithdrawalCoinFeesResponseDto


class WithdrawalCoinResponseDto(PayXRocketObject):
    code: str
    """Crypto code"""
    min_withdraw: Union[int, float] = Field(alias="minWithdraw")
    """Minimal amount for withdrawals"""
    fees: list[WithdrawalCoinFeesResponseDto]
    """Fees"""



from typing import Optional, Union

from pydantic import Field
from stollen.enums import HTTPMethod

from ..types import InvoiceDto
from .base import PayXRocketMethod


class CreateInvoice(
    PayXRocketMethod[InvoiceDto],
    http_method=HTTPMethod.POST,
    api_method="/tg-invoices",
    returning=InvoiceDto,
    response_data_key=["data"],
):
    """
    Create invoice
    """

    amount: Optional[Union[int, float]] = None
    """Invoice amount. 9 decimal places, others cut off"""
    min_payment: Optional[Union[int, float]] = Field(
        default=None, serialization_alias="minPayment"
    )
    """Min payment only for multi invoice if invoice amount is null"""
    num_payments: Union[int, float] = Field(serialization_alias="numPayments")
    """Num payments for invoice"""
    currency: str
    """Currency of transfer, info /currencies/available"""
    description: Optional[str] = None
    """Description for invoice"""
    hidden_message: Optional[str] = Field(default=None, serialization_alias="hiddenMessage")
    """Hidden message after invoice is paid"""
    comments_enabled: Optional[bool] = Field(default=None, serialization_alias="commentsEnabled")
    """Allow comments"""
    callback_url: Optional[str] = Field(default=None, serialization_alias="callbackUrl")
    """Url for Return button after invoice is paid"""
    payload: Optional[str] = None
    """Any data. Invisible to user, will be returned in callback"""
    expired_in: Optional[Union[int, float]] = Field(default=None, serialization_alias="expiredIn")
    """Invoice expire time in seconds, max 1 day, 0 - none expired"""



from stollen.enums import HTTPMethod

from ..types import PairDto
from .base import TradeXRocketMethod


class GetPairs(
    TradeXRocketMethod[list[PairDto]],
    http_method=HTTPMethod.GET,
    api_method="/pairs",
    returning=list[PairDto],
    response_data_key=["data"],
):
    """
    Returns available pairs on Rocket Exchange
    """



from .cancel_order import CancelOrder
from .create_order import CreateOrder
from .get_available_crypto_currencies import GetAvailableCryptoCurrencies
from .get_available_fiat_currencies import GetAvailableFiatCurrencies
from .get_balance import GetBalance
from .get_balances import GetBalances
from .get_crypto_rate_in_crypto import GetCryptoRateInCrypto
from .get_crypto_rate_in_fiat import GetCryptoRateInFiat
from .get_fees import GetFees
from .get_last_trades import GetLastTrades
from .get_order import GetOrder
from .get_order_book import GetOrderBook
from .get_order_book_compact import GetOrderBookCompact
from .get_order_estimate import GetOrderEstimate
from .get_orders import GetOrders
from .get_orders_by_pair import GetOrdersByPair
from .get_pair import GetPair
from .get_pairs import GetPairs
from .get_time_series import GetTimeSeries
from .get_version import GetVersion
from .get_withdrawal_fees import GetWithdrawalFees
from .get_withdrawal_status import GetWithdrawalStatus
from .withdrawal import Withdrawal

__all__ = [
    "CancelOrder",
    "CreateOrder",
    "GetAvailableCryptoCurrencies",
    "GetAvailableFiatCurrencies",
    "GetBalance",
    "GetBalances",
    "GetCryptoRateInCrypto",
    "GetCryptoRateInFiat",
    "GetFees",
    "GetLastTrades",
    "GetOrder",
    "GetOrderBook",
    "GetOrderBookCompact",
    "GetOrderEstimate",
    "GetOrders",
    "GetOrdersByPair",
    "GetPair",
    "GetPairs",
    "GetTimeSeries",
    "GetVersion",
    "GetWithdrawalFees",
    "GetWithdrawalStatus",
    "Withdrawal",
]



from typing import Optional, Union

from pydantic import Field
from stollen.enums import HTTPMethod

from ..types import WithdrawalResponseDto
from .base import TradeXRocketMethod


class Withdrawal(
    TradeXRocketMethod[WithdrawalResponseDto],
    http_method=HTTPMethod.POST,
    api_method="/account/withdrawal",
    returning=WithdrawalResponseDto,
    response_data_key=["data"],
):
    """
    Make withdrawal of funds to external wallet
    """

    network: str
    """Network code"""
    address: str
    """Withdrawal address"""
    currency: str
    """Currency code"""
    amount: Union[int, float]
    """Withdrawal amount. 9 decimal places, others cut off"""
    withdrawal_id: str = Field(serialization_alias="withdrawalId")
    """Unique withdrawal ID in your system to prevent double spends"""
    comment: Optional[str] = None
    """Withdrawal comment"""



from typing import Union

from pydantic import Field

from .base import PayXRocketObject


class SubscriptionResource(PayXRocketObject):
    id: Union[int, float]
    """Resource ID"""
    type: str
    """Resource type"""
    resource_id: str = Field(alias="resourceId")
    """Resource TG ID"""
    name: str
    """Resource TG title"""
    linked_chat: str = Field(alias="linkedChat")
    """Resource TG ID"""



from typing import Union

from pydantic import Field
from stollen.enums import HTTPMethod

from ..types import UserSubscription
from .base import PayXRocketMethod


class CheckSubscription(
    PayXRocketMethod[UserSubscription],
    http_method=HTTPMethod.POST,
    api_method="/subscriptions/check/{subscription_id}",
    returning=UserSubscription,
    response_data_key=["data"],
):
    subscription_id: str
    user_id: Union[int, float] = Field(serialization_alias="userId")
    """TG User id"""



from stollen.enums import HTTPMethod

from ..types import AccountBalancesDto
from .base import TradeXRocketMethod


class GetBalances(
    TradeXRocketMethod[AccountBalancesDto],
    http_method=HTTPMethod.GET,
    api_method="/account/balance",
    returning=AccountBalancesDto,
    response_data_key=["data"],
):
    """
    Returns current balances of you account
    """



from stollen.enums import HTTPMethod

from ..types import ShortChequeDto
from .base import PayXRocketMethod


class GetCheque(
    PayXRocketMethod[ShortChequeDto],
    http_method=HTTPMethod.GET,
    api_method="/multi-cheque/{id}",
    returning=ShortChequeDto,
    response_data_key=["data"],
):
    """
    Get multi-cheque info
    """

    id: str



from typing import Union

from pydantic import Field

from .base import TradeXRocketObject


class OrderEstimateRequestDto(TradeXRocketObject):
    pair_id: Union[int, float] = Field(alias="pairId")
    """Pair ID"""
    type: str
    """Order type"""
    execute_type: str = Field(alias="executeType")
    """Order execute type"""
    rate: Union[int, float]
    """Order rate"""
    amount: Union[int, float]
    """Amount"""
    currency: str
    """Amount currency"""



from stollen.enums import HTTPMethod

from ..types import SubscriptionInterval
from .base import PayXRocketMethod


class EditSubscriptionInterval(
    PayXRocketMethod[SubscriptionInterval],
    http_method=HTTPMethod.PUT,
    api_method="/subscriptions/{subscription_id}/interval/{interval_code}",
    returning=SubscriptionInterval,
    response_data_key=["data"],
):
    """
    Edit subscription interval
    """

    subscription_id: str
    interval_code: str
    status: str
    """Status for subscription"""



from .account_balance_dto import AccountBalanceDto
from .base import TradeXRocketObject


class AccountBalancesDto(TradeXRocketObject):
    balances: list[AccountBalanceDto]
    """Balances of you account"""



from stollen.enums import HTTPMethod

from ..types import AvailableCoinsData
from .base import PayXRocketMethod


class GetCoins(
    PayXRocketMethod[AvailableCoinsData],
    http_method=HTTPMethod.GET,
    api_method="/currencies/available",
    returning=AvailableCoinsData,
    response_data_key=["data"],
):
    """
    Returns available currencies
    """



from typing import Optional

from stollen.enums import HTTPMethod

from .base import TradeXRocketMethod


class GetVersion(
    TradeXRocketMethod[Optional[bool]],
    http_method=HTTPMethod.GET,
    api_method="/version",
    returning=Optional[bool],
    response_data_key=["success"],
):
    """
    Returns current version of API. You may use it as healthcheck
    """



from typing import Optional, Union

from stollen.enums import HTTPMethod

from ..types import LastTradesResponseDto
from .base import TradeXRocketMethod


class GetLastTrades(
    TradeXRocketMethod[list[LastTradesResponseDto]],
    http_method=HTTPMethod.GET,
    api_method="/trades/last/{pair}",
    returning=list[LastTradesResponseDto],
    response_data_key=["data"],
):
    """
    Get last trades by pair
    """

    pair: str
    limit: Optional[Union[int, float]] = None



from datetime import datetime

from .base import TradeXRocketObject


class WebhookDto(TradeXRocketObject):
    type: str
    """type of webhook sent"""
    timestamp: datetime
    """When webhook was sent"""



from datetime import datetime
from typing import Union, Optional

from pydantic import Field

from .base import PayXRocketObject


class PayInvoiceStatDto(PayXRocketObject):
    user_id: Union[int, float] = Field(alias="userId")
    """Telegram ID of user who made transaction"""
    payment_num: Union[int, float] = Field(alias="paymentNum")
    """Num of payments in transaction"""
    payment_amount: Union[int, float] = Field(alias="paymentAmount")
    """Amount of payments in transaction"""
    comment: Optional[str] = None
    """Comment on payment"""
    paid: Optional[datetime] = None
    """When transaction was paid"""



from typing import Union

from pydantic import Field

from .base import PayXRocketObject
from .set import Set
from .tg_resource import TgResource


class Cheque(PayXRocketObject):
    id: Union[int, float]
    """Cheque ID"""
    currency: str
    total: Union[int, float]
    """Total amount of cheque (this amount is charged from balance)"""
    per_user: Union[int, float] = Field(alias="perUser")
    """Amount of cheque per user"""
    users: Union[int, float]
    """Number of users that can activate your cheque"""
    password: str
    """Cheque password"""
    description: str
    """Cheque description"""
    send_notifications: bool = Field(alias="sendNotifications")
    """send notifications about cheque activation to application cheque webhook or not"""
    captcha_enabled: bool = Field(alias="captchaEnabled")
    """enable / disable cheque captcha"""
    ref_program_percents: Union[int, float] = Field(alias="refProgramPercents")
    """percentage of cheque that rewarded for referral program"""
    ref_reward_per_user: Union[int, float] = Field(alias="refRewardPerUser")
    """amount of referral user reward"""
    state: str
    """Active - cheque created and has unclaimed activations. Completed - cheque totally activated."""
    link: str
    tg_resources: list[TgResource] = Field(alias="tgResources")
    activations: Union[int, float]
    """How many times cheque is activated"""
    ref_rewards: Union[int, float] = Field(alias="refRewards")
    """How many times referral reward is payed"""
    disabled_languages: Set = Field(alias="disabledLanguages")
    """Disable languages"""
    for_premium: bool = Field(alias="forPremium")
    """Only users with Telegram Premium can activate this cheque"""
    for_new_users_only: bool = Field(alias="forNewUsersOnly")
    """Only new users can activate this cheque"""
    linked_wallet: bool = Field(alias="linkedWallet")
    """Only users with connected wallets can activate this cheque"""



from typing import Union

from .base import TradeXRocketObject


class ResponsePaginatedDto(TradeXRocketObject):
    success: bool
    """Indicate if request is successful"""
    total: Union[int, float]
    """Total times"""
    limit: Union[int, float]
    offset: Union[int, float]



from typing import Optional, Union

from pydantic import Field

from .base import TradeXRocketObject


class OrderEstimateDto(TradeXRocketObject):
    pair_id: Union[int, float] = Field(alias="pairId")
    """Pair ID"""
    in_amount: Union[int, float] = Field(alias="inAmount")
    """In amount"""
    rate: Union[int, float]
    """Order rate"""
    base_amount: Union[int, float] = Field(alias="baseAmount")
    """Base amount"""
    quote_amount: Union[int, float] = Field(alias="quoteAmount")
    """Quote amount"""
    return_amount: Optional[Union[int, float]] = Field(default=None, alias="returnAmount")
    """Return to balance, only for MARKET"""
    fee: Union[int, float]
    """Order fee"""



from stollen.enums import HTTPMethod

from ..types import FiatCurrenciesDto
from .base import TradeXRocketMethod


class GetAvailableFiatCurrencies(
    TradeXRocketMethod[FiatCurrenciesDto],
    http_method=HTTPMethod.GET,
    api_method="/rates/fiat/available",
    returning=FiatCurrenciesDto,
    response_data_key=["data"],
):
    """
    Returns available fiat currencies for exchange rates
    """



from stollen.enums import HTTPMethod

from ..types import App
from .base import PayXRocketMethod


class GetAppInfo(
    PayXRocketMethod[App],
    http_method=HTTPMethod.GET,
    api_method="/app/info",
    returning=App,
    response_data_key=["data"],
):
    """
    Returns information about your application
    """



from .cheque_status import ChequeStatus
from .invoice_status import InvoiceStatus
from .network_code import NetworkCode
from .order_execute_type import OrderExecuteType
from .order_status import OrderStatus
from .order_type import OrderType
from .subscription_interval import SubscriptionInterval
from .subscription_status import SubscriptionStatus
from .webhook_type import WebhookType
from .withdrawal_status import WithdrawalStatus

__all__ = [
    "ChequeStatus",
    "InvoiceStatus",
    "NetworkCode",
    "OrderExecuteType",
    "OrderStatus",
    "OrderType",
    "SubscriptionInterval",
    "SubscriptionStatus",
    "WebhookType",
    "WithdrawalStatus",
]



from typing import Optional, Union

from pydantic import Field
from stollen.enums import HTTPMethod

from ..types import ExchangeOrderDto
from .base import TradeXRocketMethod


class CreateOrder(
    TradeXRocketMethod[ExchangeOrderDto],
    http_method=HTTPMethod.POST,
    api_method="/orders",
    returning=ExchangeOrderDto,
    response_data_key=["data"],
):
    """
    Execute order
    """

    pair: str
    """Exchange pair"""
    type: str
    """Order type"""
    execute_type: str = Field(serialization_alias="executeType")
    """Order execute type"""
    rate: Optional[Union[int, float]] = None
    """Order rate for executeType=LIMIT"""
    amount: Union[int, float]
    """Order amount"""
    currency: str
    """Amount currency"""



from typing import Optional, Union

from pydantic import Field

from .base import TradeXRocketObject


class CreateOrderRequestDto(TradeXRocketObject):
    pair: str
    """Exchange pair"""
    type: str
    """Order type"""
    execute_type: str = Field(alias="executeType")
    """Order execute type"""
    rate: Optional[Union[int, float]] = None
    """Order rate for executeType=LIMIT"""
    amount: Union[int, float]
    """Order amount"""
    currency: str
    """Amount currency"""



from stollen.enums import HTTPMethod

from ..types import OrderBookFullDto
from .base import TradeXRocketMethod


class GetOrderBook(
    TradeXRocketMethod[OrderBookFullDto],
    http_method=HTTPMethod.GET,
    api_method="/order-book/full/{pair}",
    returning=OrderBookFullDto,
    response_data_key=["data"],
):
    """
    Returns full order book for pair
    """

    pair: str



from datetime import datetime
from typing import Optional, Union

from pydantic import Field

from .base import TradeXRocketObject


class PairDto(TradeXRocketObject):
    name: str
    """Pair name"""
    last_price: Union[int, float] = Field(alias="lastPrice")
    """Last price"""
    sell_price: Union[int, float] = Field(alias="sellPrice")
    """Best sell price"""
    buy_price: Union[int, float] = Field(alias="buyPrice")
    """Best buy price"""
    main_volume24h: Union[int, float] = Field(alias="mainVolume24h")
    """Volume in base currency, 24h"""
    quote_volume24h: Union[int, float] = Field(alias="quoteVolume24h")
    """Volume in quote currency, 24h"""
    min_main: Union[int, float] = Field(alias="minMain")
    """Min main value for exchange"""
    min_secondary: Union[int, float] = Field(alias="minSecondary")
    """Min secondary value for exchange"""
    rate_max_decimals: Union[int, float] = Field(alias="rateMaxDecimals")
    """Max decimals for rate"""
    is_open: bool = Field(alias="isOpen")
    """Indicate if trades open"""
    open24h: Union[int, float]
    """Open 24h rate"""
    main_decimals: Union[int, float] = Field(alias="mainDecimals")
    """Decimals main"""
    secondary_decimals: Union[int, float] = Field(alias="secondaryDecimals")
    """Decimals secondary"""
    open_time: Optional[datetime] = Field(default=None, alias="openTime")
    """Indicate time open trades"""



from stollen.enums import HTTPMethod

from ..types import ExchangeOrderDto
from .base import TradeXRocketMethod


class GetOrder(
    TradeXRocketMethod[ExchangeOrderDto],
    http_method=HTTPMethod.GET,
    api_method="/orders/{id}",
    returning=ExchangeOrderDto,
    response_data_key=["data"],
):
    """
    Get exchange order
    """

    id: str



import asyncio
import logging
from typing import Final

from milkpay.xrocket.pay import PayXRocket

PAY_KEY: Final[str] = "PAY_KEY_HERE"


async def main() -> None:
    logging.basicConfig(level=logging.DEBUG)
    xrocket: PayXRocket = PayXRocket(pay_key=PAY_KEY)
    try:
        for invoice in await xrocket.get_invoices():
            logging.info("Invoice #%d created at %s", invoice.id, invoice.created)
    finally:
        await xrocket.session.close()


if __name__ == "__main__":
    asyncio.run(main())



from typing import Optional, Union

from pydantic import Field

from .base import PayXRocketObject
from .subscription_interval import SubscriptionInterval
from .subscription_resource import SubscriptionResource


class Subscription(PayXRocketObject):
    id: Union[int, float]
    """Subscription ID"""
    name: str
    """Subscription name"""
    description: Optional[str] = None
    """Subscription description"""
    currency: str
    """Subscription currency"""
    link: str
    """Link to subscription"""
    interval: SubscriptionInterval
    """Subscription interval"""
    referral_percent: Union[int, float] = Field(alias="referralPercent")
    """Subscription referral percent"""
    return_url: Optional[str] = Field(default=None, alias="returnUrl")
    """Return link after payment"""
    tg_resource: Optional[SubscriptionResource] = Field(default=None, alias="tgResource")
    """TG Resource for subscription"""



from .check_subscription import CheckSubscription
from .create_cheque import CreateCheque
from .create_invoice import CreateInvoice
from .create_subscription import CreateSubscription
from .create_subscription_interval import CreateSubscriptionInterval
from .delete_cheque import DeleteCheque
from .delete_invoice import DeleteInvoice
from .delete_subscription import DeleteSubscription
from .delete_subscription_interval import DeleteSubscriptionInterval
from .edit_cheque import EditCheque
from .edit_subscription_interval import EditSubscriptionInterval
from .get_app_info import GetAppInfo
from .get_cheque import GetCheque
from .get_cheques import GetCheques
from .get_coins import GetCoins
from .get_invoice import GetInvoice
from .get_invoices import GetInvoices
from .get_subscription import GetSubscription
from .get_subscription_interval import GetSubscriptionInterval
from .get_subscriptions import GetSubscriptions
from .get_version import GetVersion
from .get_withdrawal_fees import GetWithdrawalFees
from .get_withdrawal_status import GetWithdrawalStatus
from .transfer import Transfer
from .withdrawal import Withdrawal

__all__ = [
    "CheckSubscription",
    "CreateCheque",
    "CreateInvoice",
    "CreateSubscription",
    "CreateSubscriptionInterval",
    "DeleteCheque",
    "DeleteInvoice",
    "DeleteSubscription",
    "DeleteSubscriptionInterval",
    "EditCheque",
    "EditSubscriptionInterval",
    "GetAppInfo",
    "GetCheque",
    "GetCheques",
    "GetCoins",
    "GetInvoice",
    "GetInvoices",
    "GetSubscription",
    "GetSubscriptionInterval",
    "GetSubscriptions",
    "GetVersion",
    "GetWithdrawalFees",
    "GetWithdrawalStatus",
    "Transfer",
    "Withdrawal",
]



from typing import Optional, Union

from pydantic import Field
from stollen.enums import HTTPMethod

from ..types import Subscription, SubscriptionIntervalDto
from .base import PayXRocketMethod


class CreateSubscription(
    PayXRocketMethod[Subscription],
    http_method=HTTPMethod.POST,
    api_method="/subscriptions",
    returning=Subscription,
    response_data_key=["data"],
):
    """
    Create subscription
    """

    name: Optional[str] = None
    """Subscription name, view in bot"""
    description: Optional[str] = None
    """Subscription description, view in bot"""
    currency: str
    """Subscription currency"""
    interval: list[SubscriptionIntervalDto]
    """Subscription interval"""
    tg_resource: Optional[str] = Field(default=None, serialization_alias="tgResource")
    """Subscription TG resource"""
    referral_percent: Union[int, float] = Field(serialization_alias="referralPercent")
    """Subscription referral percent"""
    return_url: Optional[str] = Field(default=None, serialization_alias="returnUrl")
    """Return link after payment"""



from typing import Union

from .base import PayXRocketObject


class WithdrawFeeNetworkFeeDto(PayXRocketObject):
    fee: Union[int, float]
    """Fee amount"""
    currency: str
    """Withdraw fee currency"""



from stollen.enums import HTTPMethod

from ..types import UserFeesResponseDto
from .base import TradeXRocketMethod


class GetFees(
    TradeXRocketMethod[UserFeesResponseDto],
    http_method=HTTPMethod.GET,
    api_method="/account/fees",
    returning=UserFeesResponseDto,
    response_data_key=["data"],
):
    """
    Returns current user fees
    """



from typing import Any, Generator, Generic, Optional, TypeVar

from .base import PayXRocketObject

T = TypeVar("T", bound=Any)


class Pagination(PayXRocketObject, Generic[T]):
    total: int
    limit: Optional[int] = None
    offset: Optional[int] = None
    results: list[T]

    def __iter__(self) -> Generator[T, None, None]:
        yield from self.results



from typing import Union

from .base import PayXRocketObject


class SubscriptionIntervalDto(PayXRocketObject):
    interval: str
    """Interval for subscription"""
    amount: Union[int, float]
    """Cost subscription for current interval in currency"""
    status: str
    """Status for subscription"""



from typing import Optional

from pydantic import Field

from .base import PayXRocketObject


class UpdateChequeDto(PayXRocketObject):
    password: Optional[str] = None
    """Password for cheque"""
    description: Optional[str] = None
    """Description for cheque"""
    send_notifications: Optional[bool] = Field(default=None, alias="sendNotifications")
    """Send notifications about activations"""
    enable_captcha: Optional[bool] = Field(default=None, alias="enableCaptcha")
    """Enable captcha"""
    telegram_resources_ids: Optional[list[str]] = Field(default=None, alias="telegramResourcesIds")
    """IDs of telegram resources (groups, channels, private groups)"""
    for_premium: Optional[bool] = Field(default=None, alias="forPremium")
    """Only users with Telegram Premium can activate this cheque"""
    linked_wallet: Optional[bool] = Field(default=None, alias="linkedWallet")
    """Only users with linked wallet can activate this cheque"""
    disabled_languages: Optional[list[str]] = Field(default=None, alias="disabledLanguages")
    """Disable languages"""



from typing import Union

from pydantic import Field

from .base import TradeXRocketObject


class RateCryptoFiatDto(TradeXRocketObject):
    crypto_currency: str = Field(alias="cryptoCurrency")
    """Crypto currency"""
    fiat_currency: str = Field(alias="fiatCurrency")
    """Fiat currency"""
    rate: Union[int, float]
    """Crypto rate in fiat"""



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Optional, Union

from stollen import Stollen
from stollen.requests.fields import Header, RequestField

from .exceptions import TradeXRocketError, UnknownAPIKeyError

if TYPE_CHECKING:
    from .types import (
        AccountBalanceDto,
        AccountBalancesDto,
        CryptoCurrenciesResponseDto,
        ExchangeOrderDto,
        FiatCurrenciesDto,
        LastTradesResponseDto,
        OrderBookFullDto,
        OrderEstimateDto,
        Pagination,
        PairDto,
        RateCryptoCryptoDto,
        RateCryptoFiatDto,
        TimeSeriesDto,
        UserFeesResponseDto,
        WithdrawalCoinResponseDto,
        WithdrawalResponseDto,
    )


class TradeXRocket(Stollen):
    trade_key: Optional[str]
    production: bool

    def __init__(
        self,
        trade_key: Optional[str] = None,
        production: bool = True,
        **stollen_kwargs: Any,
    ) -> None:
        self.trade_key = trade_key
        self.production = production

        fields: list[RequestField] = []
        if trade_key is not None:
            fields.append(Header(name="Rocket-Exchange-Key", value=trade_key))

        subdomain: str = "trade" if production else "dev-trade"
        super().__init__(
            base_url=f"https://{subdomain}.xrocket.tg",
            error_message_key=["message"],
            global_request_fields=fields,
            general_error_class=TradeXRocketError,
            error_codes={403: UnknownAPIKeyError},
            **stollen_kwargs,
        )

    async def get_version(self) -> Optional[bool]:
        """
        Returns current version of API. You may use it as healthcheck
        """

        from .methods import GetVersion

        call: GetVersion = GetVersion()

        return await self(call)

    async def get_balances(self) -> AccountBalancesDto:
        """
        Returns current balances of you account
        """

        from .methods import GetBalances

        call: GetBalances = GetBalances()

        return await self(call)

    async def get_fees(self) -> UserFeesResponseDto:
        """
        Returns current user fees
        """

        from .methods import GetFees

        call: GetFees = GetFees()

        return await self(call)

    async def get_balance(
        self,
        *,
        coin: str,
    ) -> AccountBalanceDto:
        """
        Returns current balance of you account for coin
        """

        from .methods import GetBalance

        call: GetBalance = GetBalance(
            coin=coin,
        )

        return await self(call)

    async def get_withdrawal_fees(
        self,
        *,
        currency: Optional[str] = None,
    ) -> list[WithdrawalCoinResponseDto]:
        """
        Returns withdrawal fees
        """

        from .methods import GetWithdrawalFees

        call: GetWithdrawalFees = GetWithdrawalFees(
            currency=currency,
        )

        return await self(call)

    async def withdrawal(
        self,
        *,
        network: str,
        address: str,
        currency: str,
        amount: Union[int, float],
        withdrawal_id: str,
        comment: Optional[str] = None,
    ) -> WithdrawalResponseDto:
        """
        Make withdrawal of funds to external wallet
        """

        from .methods import Withdrawal

        call: Withdrawal = Withdrawal(
            network=network,
            address=address,
            currency=currency,
            amount=amount,
            withdrawal_id=withdrawal_id,
            comment=comment,
        )

        return await self(call)

    async def get_withdrawal_status(
        self,
        *,
        withdrawal_id: str,
    ) -> WithdrawalResponseDto:
        """
        Returns withdrawal status
        """

        from .methods import GetWithdrawalStatus

        call: GetWithdrawalStatus = GetWithdrawalStatus(
            withdrawal_id=withdrawal_id,
        )

        return await self(call)

    async def get_pairs(self) -> list[PairDto]:
        """
        Returns available pairs on Rocket Exchange
        """

        from .methods import GetPairs

        call: GetPairs = GetPairs()

        return await self(call)

    async def get_pair(
        self,
        *,
        pair: str,
    ) -> PairDto:
        """
        Return pair by base/quote coin on Rocket Exchange
        """

        from .methods import GetPair

        call: GetPair = GetPair(
            pair=pair,
        )

        return await self(call)

    async def get_available_fiat_currencies(self) -> FiatCurrenciesDto:
        """
        Returns available fiat currencies for exchange rates
        """

        from .methods import GetAvailableFiatCurrencies

        call: GetAvailableFiatCurrencies = GetAvailableFiatCurrencies()

        return await self(call)

    async def get_available_crypto_currencies(self) -> CryptoCurrenciesResponseDto:
        """
        Returns available crypto currencies for exchange rates
        """

        from .methods import GetAvailableCryptoCurrencies

        call: GetAvailableCryptoCurrencies = GetAvailableCryptoCurrencies()

        return await self(call)

    async def get_crypto_rate_in_fiat(
        self,
        *,
        crypto: str,
        fiat: str,
    ) -> RateCryptoFiatDto:
        """
        Returns rate crypto currency in fiat
        """

        from .methods import GetCryptoRateInFiat

        call: GetCryptoRateInFiat = GetCryptoRateInFiat(
            crypto=crypto,
            fiat=fiat,
        )

        return await self(call)

    async def get_crypto_rate_in_crypto(
        self,
        *,
        base: str,
        quote: str,
    ) -> RateCryptoCryptoDto:
        """
        Returns rate crypto currency in crypto
        """

        from .methods import GetCryptoRateInCrypto

        call: GetCryptoRateInCrypto = GetCryptoRateInCrypto(
            base=base,
            quote=quote,
        )

        return await self(call)

    async def get_time_series(
        self,
        *,
        pair: str,
        start_date: str,
        end_date: str,
        period: str,
    ) -> TimeSeriesDto:
        """
        Get timeseries for pair
        """

        from .methods import GetTimeSeries

        call: GetTimeSeries = GetTimeSeries(
            pair=pair,
            start_date=start_date,
            end_date=end_date,
            period=period,
        )

        return await self(call)

    async def get_order_book(
        self,
        *,
        pair: str,
    ) -> OrderBookFullDto:
        """
        Returns full order book for pair
        """

        from .methods import GetOrderBook

        call: GetOrderBook = GetOrderBook(
            pair=pair,
        )

        return await self(call)

    async def get_order_book_compact(
        self,
        *,
        pair: str,
    ) -> OrderBookFullDto:
        """
        Returns full order book for pair
        """

        from .methods import GetOrderBookCompact

        call: GetOrderBookCompact = GetOrderBookCompact(
            pair=pair,
        )

        return await self(call)

    async def get_orders(
        self,
        *,
        limit: Optional[Union[int, float]] = None,
        offset: Optional[Union[int, float]] = None,
        only_active: bool,
    ) -> Pagination[ExchangeOrderDto]:
        """
        Get list of exchange orders
        """

        from .methods import GetOrders

        call: GetOrders = GetOrders(
            limit=limit,
            offset=offset,
            only_active=only_active,
        )

        return await self(call)

    async def create_order(
        self,
        *,
        pair: str,
        type_: str,
        execute_type: str,
        rate: Optional[Union[int, float]] = None,
        amount: Union[int, float],
        currency: str,
    ) -> ExchangeOrderDto:
        """
        Execute order
        """

        from .methods import CreateOrder

        call: CreateOrder = CreateOrder(
            pair=pair,
            type=type_,
            execute_type=execute_type,
            rate=rate,
            amount=amount,
            currency=currency,
        )

        return await self(call)

    async def get_orders_by_pair(
        self,
        *,
        pair: str,
        limit: Optional[Union[int, float]] = None,
        offset: Optional[Union[int, float]] = None,
        only_active: bool,
    ) -> Pagination[ExchangeOrderDto]:
        """
        Get list of exchange orders by pair
        """

        from .methods import GetOrdersByPair

        call: GetOrdersByPair = GetOrdersByPair(
            pair=pair,
            limit=limit,
            offset=offset,
            only_active=only_active,
        )

        return await self(call)

    async def get_order(
        self,
        *,
        id_: str,
    ) -> ExchangeOrderDto:
        """
        Get exchange order
        """

        from .methods import GetOrder

        call: GetOrder = GetOrder(
            id=id_,
        )

        return await self(call)

    async def cancel_order(
        self,
        *,
        id_: str,
    ) -> Optional[bool]:
        """
        Cancel exchange order
        """

        from .methods import CancelOrder

        call: CancelOrder = CancelOrder(
            id=id_,
        )

        return await self(call)

    async def get_order_estimate(
        self,
        *,
        pair_id: Union[int, float],
        type_: str,
        execute_type: str,
        rate: Union[int, float],
        amount: Union[int, float],
        currency: str,
    ) -> OrderEstimateDto:
        """
        Returns order estimate
        """

        from .methods import GetOrderEstimate

        call: GetOrderEstimate = GetOrderEstimate(
            pair_id=pair_id,
            type=type_,
            execute_type=execute_type,
            rate=rate,
            amount=amount,
            currency=currency,
        )

        return await self(call)

    async def get_last_trades(
        self,
        *,
        pair: str,
        limit: Optional[Union[int, float]] = None,
    ) -> list[LastTradesResponseDto]:
        """
        Get last trades by pair
        """

        from .methods import GetLastTrades

        call: GetLastTrades = GetLastTrades(
            pair=pair,
            limit=limit,
        )

        return await self(call)



from stollen.enums import HTTPMethod

from ..types import TimeSeriesDto
from .base import TradeXRocketMethod


class GetTimeSeries(
    TradeXRocketMethod[TimeSeriesDto],
    http_method=HTTPMethod.GET,
    api_method="/time-series/{pair}",
    returning=TimeSeriesDto,
    response_data_key=["data"],
):
    """
    Get timeseries for pair
    """

    pair: str
    start_date: str
    end_date: str
    period: str



from typing import Optional, Union

from stollen.enums import HTTPMethod

from ..types import Pagination, Subscription
from .base import PayXRocketMethod


class GetSubscriptions(
    PayXRocketMethod[Pagination[Subscription]],
    http_method=HTTPMethod.GET,
    api_method="/subscriptions",
    returning=Pagination[Subscription],
    response_data_key=["data"],
):
    """
    Get list of subscription
    """

    limit: Optional[Union[int, float]] = None
    offset: Optional[Union[int, float]] = None



from .available_coins_data import AvailableCoinsData
from .base import PayXRocketObject


class AvailableCoins(PayXRocketObject):
    success: bool
    """indicate if request is successful"""
    data: AvailableCoinsData



from __future__ import annotations

from hashlib import sha256
from hmac import HMAC
from typing import TYPE_CHECKING, Any, Optional, Union

from stollen import Stollen
from stollen.requests.fields import Header, RequestField

from .exceptions import PayXRocketError, UnknownAPIKeyError

if TYPE_CHECKING:
    from .types import (
        App,
        AvailableCoinsData,
        Cheque,
        FullInvoiceDto,
        InvoiceDto,
        Pagination,
        ShortChequeDto,
        Subscription,
        SubscriptionInterval,
        SubscriptionIntervalDto,
        TransferDto,
        UserSubscription,
        WithdrawalCoinResponseDto,
        WithdrawalDto,
    )


class PayXRocket(Stollen):
    pay_key: Optional[str]
    production: bool

    def __init__(
        self,
        pay_key: Optional[str] = None,
        production: bool = True,
        **stollen_kwargs: Any,
    ) -> None:
        self.pay_key = pay_key
        self.production = production

        fields: list[RequestField] = []
        if pay_key is not None:
            fields.append(Header(name="Rocket-Pay-Key", value=pay_key))

        subdomain: str = "pay" if production else "dev-pay"
        super().__init__(
            base_url=f"https://{subdomain}.xrocket.tg",
            error_message_key=["message"],
            global_request_fields=fields,
            general_error_class=PayXRocketError,
            error_codes={403: UnknownAPIKeyError},
            **stollen_kwargs,
        )

    def check_signature(self, body_text: str, signature: str) -> bool:
        if self.pay_key is None:
            raise ValueError("Pay key is not set")
        hmac: HMAC = HMAC(
            key=sha256(string=self.pay_key.encode("UTF-8")).digest(),
            msg=body_text.encode("UTF-8"),
            digestmod=sha256,
        )
        return hmac.hexdigest() == signature

    async def get_version(self) -> Optional[bool]:
        """
        Returns current version of API. You may use it as healthcheck
        """

        from .methods import GetVersion

        call: GetVersion = GetVersion()

        return await self(call)

    async def create_cheque(
        self,
        *,
        currency: str,
        cheque_per_user: Union[int, float],
        users_number: Union[int, float],
        ref_program: Union[int, float],
        password: Optional[str] = None,
        description: Optional[str] = None,
        send_notifications: Optional[bool] = None,
        enable_captcha: Optional[bool] = None,
        telegram_resources_ids: Optional[list[str]] = None,
        for_premium: Optional[bool] = None,
        linked_wallet: Optional[bool] = None,
        disabled_languages: Optional[list[str]] = None,
    ) -> Cheque:
        """
        Create multi-cheque
        """

        from .methods import CreateCheque

        call: CreateCheque = CreateCheque(
            currency=currency,
            cheque_per_user=cheque_per_user,
            users_number=users_number,
            ref_program=ref_program,
            password=password,
            description=description,
            send_notifications=send_notifications,
            enable_captcha=enable_captcha,
            telegram_resources_ids=telegram_resources_ids,
            for_premium=for_premium,
            linked_wallet=linked_wallet,
            disabled_languages=disabled_languages,
        )

        return await self(call)

    async def get_cheques(
        self,
        *,
        limit: Optional[Union[int, float]] = None,
        offset: Optional[Union[int, float]] = None,
    ) -> Pagination[ShortChequeDto]:
        """
        Get list of multi-cheques
        """

        from .methods import GetCheques

        call: GetCheques = GetCheques(
            limit=limit,
            offset=offset,
        )

        return await self(call)

    async def get_cheque(
        self,
        *,
        id_: str,
    ) -> ShortChequeDto:
        """
        Get multi-cheque info
        """

        from .methods import GetCheque

        call: GetCheque = GetCheque(
            id=id_,
        )

        return await self(call)

    async def edit_cheque(
        self,
        *,
        id_: str,
        password: Optional[str] = None,
        description: Optional[str] = None,
        send_notifications: Optional[bool] = None,
        enable_captcha: Optional[bool] = None,
        telegram_resources_ids: Optional[list[str]] = None,
        for_premium: Optional[bool] = None,
        linked_wallet: Optional[bool] = None,
        disabled_languages: Optional[list[str]] = None,
    ) -> Cheque:
        """
        Edit multi-cheque
        """

        from .methods import EditCheque

        call: EditCheque = EditCheque(
            id=id_,
            password=password,
            description=description,
            send_notifications=send_notifications,
            enable_captcha=enable_captcha,
            telegram_resources_ids=telegram_resources_ids,
            for_premium=for_premium,
            linked_wallet=linked_wallet,
            disabled_languages=disabled_languages,
        )

        return await self(call)

    async def delete_cheque(
        self,
        *,
        id_: str,
    ) -> Optional[bool]:
        """
        Delete multi-cheque
        """

        from .methods import DeleteCheque

        call: DeleteCheque = DeleteCheque(
            id=id_,
        )

        return await self(call)

    async def get_app_info(self) -> App:
        """
        Returns information about your application
        """

        from .methods import GetAppInfo

        call: GetAppInfo = GetAppInfo()

        return await self(call)

    async def transfer(
        self,
        *,
        tg_user_id: Union[int, float],
        currency: str,
        amount: Union[int, float],
        transfer_id: str,
        description: Optional[str] = None,
    ) -> TransferDto:
        """
        Make transfer of funds to another user
        """

        from .methods import Transfer

        call: Transfer = Transfer(
            tg_user_id=tg_user_id,
            currency=currency,
            amount=amount,
            transfer_id=transfer_id,
            description=description,
        )

        return await self(call)

    async def withdrawal(
        self,
        *,
        network: str,
        address: str,
        currency: str,
        amount: Union[int, float],
        withdrawal_id: str,
        comment: Optional[str] = None,
    ) -> WithdrawalDto:
        """
        Make withdrawal of funds to external wallet
        """

        from .methods import Withdrawal

        call: Withdrawal = Withdrawal(
            network=network,
            address=address,
            currency=currency,
            amount=amount,
            withdrawal_id=withdrawal_id,
            comment=comment,
        )

        return await self(call)

    async def get_withdrawal_status(
        self,
        *,
        withdrawal_id: str,
    ) -> WithdrawalDto:
        """
        Returns withdrawal status
        """

        from .methods import GetWithdrawalStatus

        call: GetWithdrawalStatus = GetWithdrawalStatus(
            withdrawal_id=withdrawal_id,
        )

        return await self(call)

    async def get_withdrawal_fees(
        self,
        *,
        currency: Optional[str] = None,
    ) -> list[WithdrawalCoinResponseDto]:
        """
        Returns withdrawal fees
        """

        from .methods import GetWithdrawalFees

        call: GetWithdrawalFees = GetWithdrawalFees(
            currency=currency,
        )

        return await self(call)

    async def create_invoice(
        self,
        *,
        amount: Optional[Union[int, float]] = None,
        min_payment: Optional[Union[int, float]] = None,
        num_payments: Union[int, float],
        currency: str,
        description: Optional[str] = None,
        hidden_message: Optional[str] = None,
        comments_enabled: Optional[bool] = None,
        callback_url: Optional[str] = None,
        payload: Optional[str] = None,
        expired_in: Optional[Union[int, float]] = None,
    ) -> InvoiceDto:
        """
        Create invoice
        """

        from .methods import CreateInvoice

        call: CreateInvoice = CreateInvoice(
            amount=amount,
            min_payment=min_payment,
            num_payments=num_payments,
            currency=currency,
            description=description,
            hidden_message=hidden_message,
            comments_enabled=comments_enabled,
            callback_url=callback_url,
            payload=payload,
            expired_in=expired_in,
        )

        return await self(call)

    async def get_invoices(
        self,
        *,
        limit: Optional[Union[int, float]] = None,
        offset: Optional[Union[int, float]] = None,
    ) -> Pagination[InvoiceDto]:
        """
        Get list of invoices
        """

        from .methods import GetInvoices

        call: GetInvoices = GetInvoices(
            limit=limit,
            offset=offset,
        )

        return await self(call)

    async def get_invoice(
        self,
        *,
        id_: str,
    ) -> FullInvoiceDto:
        """
        Get invoice info
        """

        from .methods import GetInvoice

        call: GetInvoice = GetInvoice(
            id=id_,
        )

        return await self(call)

    async def delete_invoice(
        self,
        *,
        id_: str,
    ) -> Optional[bool]:
        """
        Delete invoice
        """

        from .methods import DeleteInvoice

        call: DeleteInvoice = DeleteInvoice(
            id=id_,
        )

        return await self(call)

    async def get_coins(self) -> AvailableCoinsData:
        """
        Returns available currencies
        """

        from .methods import GetCoins

        call: GetCoins = GetCoins()

        return await self(call)

    async def create_subscription(
        self,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        currency: str,
        interval: list[SubscriptionIntervalDto],
        tg_resource: Optional[str] = None,
        referral_percent: Union[int, float],
        return_url: Optional[str] = None,
    ) -> Subscription:
        """
        Create subscription
        """

        from .methods import CreateSubscription

        call: CreateSubscription = CreateSubscription(
            name=name,
            description=description,
            currency=currency,
            interval=interval,
            tg_resource=tg_resource,
            referral_percent=referral_percent,
            return_url=return_url,
        )

        return await self(call)

    async def get_subscriptions(
        self,
        *,
        limit: Optional[Union[int, float]] = None,
        offset: Optional[Union[int, float]] = None,
    ) -> Pagination[Subscription]:
        """
        Get list of subscription
        """

        from .methods import GetSubscriptions

        call: GetSubscriptions = GetSubscriptions(
            limit=limit,
            offset=offset,
        )

        return await self(call)

    async def get_subscription(
        self,
        *,
        subscription_id: str,
    ) -> Subscription:
        """
        Get subscription info
        """

        from .methods import GetSubscription

        call: GetSubscription = GetSubscription(
            subscription_id=subscription_id,
        )

        return await self(call)

    async def delete_subscription(
        self,
        *,
        subscription_id: str,
    ) -> Optional[bool]:
        """
        Delete subscription
        """

        from .methods import DeleteSubscription

        call: DeleteSubscription = DeleteSubscription(
            subscription_id=subscription_id,
        )

        return await self(call)

    async def check_subscription(
        self,
        *,
        subscription_id: str,
        user_id: Union[int, float],
    ) -> UserSubscription:
        from .methods import CheckSubscription

        call: CheckSubscription = CheckSubscription(
            subscription_id=subscription_id,
            user_id=user_id,
        )

        return await self(call)

    async def get_subscription_interval(
        self,
        *,
        subscription_id: str,
        interval_code: str,
    ) -> SubscriptionInterval:
        """
        Get subscription interval info
        """

        from .methods import GetSubscriptionInterval

        call: GetSubscriptionInterval = GetSubscriptionInterval(
            subscription_id=subscription_id,
            interval_code=interval_code,
        )

        return await self(call)

    async def edit_subscription_interval(
        self,
        *,
        subscription_id: str,
        interval_code: str,
        status: str,
    ) -> SubscriptionInterval:
        """
        Edit subscription interval
        """

        from .methods import EditSubscriptionInterval

        call: EditSubscriptionInterval = EditSubscriptionInterval(
            subscription_id=subscription_id,
            interval_code=interval_code,
            status=status,
        )

        return await self(call)

    async def delete_subscription_interval(
        self,
        *,
        subscription_id: str,
        interval_code: str,
    ) -> SubscriptionInterval:
        """
        Delete subscription interval
        """

        from .methods import DeleteSubscriptionInterval

        call: DeleteSubscriptionInterval = DeleteSubscriptionInterval(
            subscription_id=subscription_id,
            interval_code=interval_code,
        )

        return await self(call)

    async def create_subscription_interval(
        self,
        *,
        subscription_id: str,
        interval: str,
        amount: Union[int, float],
        status: str,
    ) -> SubscriptionInterval:
        """
        Create subscription interval
        """

        from .methods import CreateSubscriptionInterval

        call: CreateSubscriptionInterval = CreateSubscriptionInterval(
            subscription_id=subscription_id,
            interval=interval,
            amount=amount,
            status=status,
        )

        return await self(call)



from typing import Union

from .base import TradeXRocketObject


class WithdrawFeeNetworkFeeDto(TradeXRocketObject):
    fee: Union[int, float]
    """Fee amount"""
    currency: str
    """Withdraw fee currency"""



from typing import Union

from pydantic import Field

from .base import TradeXRocketObject


class OrderBookOrderDto(TradeXRocketObject):
    type: str
    """Order type"""
    rate: Union[int, float]
    """Order rate"""
    value: Union[int, float]
    """Order base value"""
    quote_value: Union[int, float] = Field(alias="quoteValue")
    """Order quote value"""



from stollen.enums import HTTPMethod

from ..types import WithdrawalResponseDto
from .base import TradeXRocketMethod


class GetWithdrawalStatus(
    TradeXRocketMethod[WithdrawalResponseDto],
    http_method=HTTPMethod.GET,
    api_method="/account/withdrawal/status/{withdrawal_id}",
    returning=WithdrawalResponseDto,
    response_data_key=["data"],
):
    """
    Returns withdrawal status
    """

    withdrawal_id: str



from typing import Union

from pydantic import Field

from .base import TradeXRocketObject


class UserFeesResponseDto(TradeXRocketObject):
    buy_taker: Union[int, float] = Field(alias="buyTaker")
    """Buy taker fee"""
    sell_taker: Union[int, float] = Field(alias="sellTaker")
    """Sell taker fee"""
    buy_maker: Union[int, float] = Field(alias="buyMaker")
    """Buy maker fee"""
    sell_maker: Union[int, float] = Field(alias="sellMaker")
    """Sell maker fee"""



from .base import PayXRocketObject
from .withdraw_fee_network_dto import WithdrawFeeNetworkDto


class WithdrawFeeDto(PayXRocketObject):
    currency: str
    """ID of main currency for token"""
    networks: list[WithdrawFeeNetworkDto]
    """Fee for withdraw"""



from typing import Union

from pydantic import Field

from .base import TradeXRocketObject
from .order_book_order_dto import OrderBookOrderDto


class OrderBookFullDto(TradeXRocketObject):
    pair: str
    """Pair name"""
    sell_count: Union[int, float] = Field(alias="sellCount")
    """Count sell orders"""
    buy_count: Union[int, float] = Field(alias="buyCount")
    """Count buy orders"""
    sell: list[OrderBookOrderDto]
    """List of sell orders"""
    buy: list[OrderBookOrderDto]
    """List of buy orders"""



from stollen.enums import HTTPMethod

from ..types import PairDto
from .base import TradeXRocketMethod


class GetPair(
    TradeXRocketMethod[PairDto],
    http_method=HTTPMethod.GET,
    api_method="/pairs/{pair}",
    returning=PairDto,
    response_data_key=["data"],
):
    """
    Return pair by base/quote coin on Rocket Exchange
    """

    pair: str



from typing import Union

from pydantic import Field

from .base import TradeXRocketObject


class RateCryptoCryptoDto(TradeXRocketObject):
    base_currency: str = Field(alias="baseCurrency")
    """From crypto currency"""
    quote_currency: str = Field(alias="quoteCurrency")
    """To crypto currency"""
    rate: Union[int, float]
    """Crypto rate in crypto"""



from typing import Optional, Union

from stollen.enums import HTTPMethod

from ..types import InvoiceDto, Pagination
from .base import PayXRocketMethod


class GetInvoices(
    PayXRocketMethod[Pagination[InvoiceDto]],
    http_method=HTTPMethod.GET,
    api_method="/tg-invoices",
    returning=Pagination[InvoiceDto],
    response_data_key=["data"],
):
    """
    Get list of invoices
    """

    limit: Optional[Union[int, float]] = None
    offset: Optional[Union[int, float]] = None
